@@ -19,6 +19,7 @@ use crate::temporal::BPMDesc;
 use crate::timbral::{SpectralDesc, ZeroCrossingRateDesc};
 use crate::{BlissError, SAMPLE_RATE};
 use ::log::warn;
+use core::ops::Index;
 use crossbeam::thread;
 use ffmpeg_next::codec::threading::{Config, Type as ThreadingType};
 use ffmpeg_next::software::resampling::context::Context;
@@ -30,10 +31,14 @@ use ffmpeg_next::util::format::sample::{Sample, Type};
 use ffmpeg_next::util::frame::audio::Audio;
 use ffmpeg_next::util::log;
 use ffmpeg_next::util::log::level::Level;
-use ndarray::{arr1, Array};
+use ndarray::{arr1, Array, Array1};
+use std::convert::TryInto;
+use std::fmt;
 use std::sync::mpsc;
 use std::sync::mpsc::Receiver;
 use std::thread as std_thread;
+use strum::{EnumCount, IntoEnumIterator};
+use strum_macros::{EnumCount, EnumIter};
 
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 #[derive(Default, Debug, PartialEq, Clone)]
@@ -52,14 +57,126 @@ pub struct Song {
     pub track_number: String,
     /// Song's genre, read from the metadata (`""` if empty)
     pub genre: String,
-    /// Vec containing analysis, in order: tempo, zero-crossing rate,
-    /// mean spectral centroid, std deviation spectral centroid,
-    /// mean spectral rolloff, std deviation spectral rolloff
-    /// mean spectral_flatness, std deviation spectral flatness,
-    /// mean loudness, std deviation loudness, chroma interval feature 1 to 10.
+    /// bliss analysis results
+    pub analysis: Analysis,
+}
+
+#[derive(Debug, EnumIter, EnumCount)]
+/// Indexes different fields of an [Analysis](Song::analysis).
+///
+/// * Example:
+/// ```no_run
+/// use bliss_audio::{AnalysisIndex, BlissError, Song};
+///
+/// fn main() -> Result<(), BlissError> {
+///     let song = Song::new("path/to/song")?;
+///     println!("{}", song.analysis[AnalysisIndex::Tempo]);
+///     Ok(())
+/// }
+/// ```
+///
+/// Prints the tempo value of an analysis.
+///
+/// Note that this should mostly be used for debugging / distance metric
+/// customization purposes.
+#[allow(missing_docs)]
+pub enum AnalysisIndex {
+    Tempo,
+    Zcr,
+    MeanSpectralCentroid,
+    StdDeviationSpectralCentroid,
+    MeanSpectralRolloff,
+    StdDeviationSpectralRolloff,
+    MeanSpectralFlatness,
+    StdDeviationSpectralFlatness,
+    MeanLoudness,
+    StdDeviationLoudness,
+    Chroma1,
+    Chroma2,
+    Chroma3,
+    Chroma4,
+    Chroma5,
+    Chroma6,
+    Chroma7,
+    Chroma8,
+    Chroma9,
+    Chroma10,
+}
+const NUMBER_FEATURES: usize = AnalysisIndex::COUNT;
+
+#[derive(Default, PartialEq, Clone, Copy)]
+/// Object holding the results of the song's analysis.
+///
+/// Only use it if you want to have an in-depth look of what is
+/// happening behind the scene, or make a distance metric yourself.
+///
+/// Under the hood, it is just an array of f32 holding different numeric
+/// features.
+///
+/// For more info on the different features, build the
+/// documentation with private items included using
+/// `cargo doc --document-private-items`, and / or read up
+/// [this document](https://lelele.io/thesis.pdf), that contains a description
+/// on most of the features, except the chroma ones, which are documented
+/// directly in this code.
+pub struct Analysis {
+    internal_analysis: [f32; NUMBER_FEATURES],
+}
+
+impl Index<AnalysisIndex> for Analysis {
+    type Output = f32;
+
+    fn index(&self, index: AnalysisIndex) -> &f32 {
+        &self.internal_analysis[index as usize]
+    }
+}
+
+impl fmt::Debug for Analysis {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        let mut debug_struct = f.debug_struct("Analysis");
+        for feature in AnalysisIndex::iter() {
+            debug_struct.field(&format!("{:?}", feature), &self[feature]);
+        }
+        debug_struct.finish()?;
+        f.write_str(&format!(" /* {:?} */", &self.to_vec()))
+    }
+}
+
+impl Analysis {
+    pub(crate) fn new(analysis: [f32; NUMBER_FEATURES]) -> Analysis {
+        Analysis {
+            internal_analysis: analysis,
+        }
+    }
+
+    /// Return an ndarray `arr1`.
     ///
-    /// All the numbers are between -1 and 1.
-    pub analysis: Vec<f32>,
+    /// Particularly useful if you want to make a custom distance metric.
+    pub fn to_arr1(&self) -> Array1<f32> {
+        arr1(&self.internal_analysis)
+    }
+
+    #[allow(dead_code)]
+    pub(crate) fn to_vec(&self) -> Vec<f32> {
+        self.internal_analysis.to_vec()
+    }
+
+    /// Return the [euclidean
+    /// distance](https://en.wikipedia.org/wiki/Euclidean_distance#Higher_dimensions)
+    /// between two analysis.
+    ///
+    /// Note that it is usually easier to just use [`song.distance(song2)`](Song::distance)
+    /// (which calls this function in turn).
+    pub fn distance(&self, other: &Self) -> f32 {
+        let a1 = self.to_arr1();
+        let a2 = other.to_arr1();
+        // Could be any square symmetric positive semi-definite matrix;
+        // just no metric learning has been done yet.
+        // See https://lelele.io/thesis.pdf chapter 4.
+        let m = Array::eye(NUMBER_FEATURES);
+
+        (self.to_arr1() - &a2).dot(&m).dot(&(&a1 - &a2)).sqrt()
+    }
 }
 
 impl Song {
@@ -71,21 +188,14 @@ impl Song {
     /// (e.g. if song1.distance(song2) < song1.distance(song3), then song1 is
     /// closer to song2 than it is to song3.
     pub fn distance(&self, other: &Self) -> f32 {
-        let a1 = arr1(&self.analysis.to_vec());
-        let a2 = arr1(&other.analysis.to_vec());
-        // Could be any square symmetric positive semi-definite matrix;
-        // just no metric learning has been done yet.
-        // See https://lelele.io/thesis.pdf chapter 4.
-        let m = Array::eye(self.analysis.len());
-
-        (arr1(&self.analysis) - &a2).dot(&m).dot(&(&a1 - &a2))
+        self.analysis.distance(&other.analysis)
     }
 
     /// Returns a decoded Song given a file path, or an error if the song
     /// could not be analyzed for some reason.
     ///
     /// # Arguments
-    /// 
+    ///
     /// * `path` - A string holding a valid file path to a valid audio file.
     ///
     /// # Errors
@@ -122,7 +232,7 @@ impl Song {
      * Useful in the rare cases where the full song is not
      * completely available.
      **/
-    fn analyse(sample_array: Vec<f32>) -> Result<Vec<f32>, BlissError> {
+    fn analyse(sample_array: Vec<f32>) -> Result<Analysis, BlissError> {
         let largest_window = vec![
             BPMDesc::WINDOW_SIZE,
             ChromaDesc::WINDOW_SIZE,
@@ -207,7 +317,14 @@ impl Song {
             result.extend_from_slice(&flatness);
             result.extend_from_slice(&loudness);
             result.extend_from_slice(&chroma);
-            Ok(result)
+            let array: [f32; NUMBER_FEATURES] = result.try_into().map_err(|_| {
+                BlissError::AnalysisError(
+                    "Too many or too little features were provided at the end of
+                        the analysis."
+                        .to_string(),
+                )
+            })?;
+            Ok(Analysis::new(array))
         })
         .unwrap()
     }
@@ -226,9 +343,7 @@ impl Song {
             let stream = format
                 .streams()
                 .find(|s| s.codec().medium() == ffmpeg::media::Type::Audio)
-                .ok_or_else(|| BlissError::DecodingError(String::from(
-                    "No audio stream found.",
-                )))?;
+                .ok_or_else(|| BlissError::DecodingError(String::from("No audio stream found.")))?;
             stream.codec().set_threading(Config {
                 kind: ThreadingType::Frame,
                 count: 0,
@@ -477,7 +592,7 @@ mod tests {
             -0.9820945,
             -0.95968974,
         ];
-        for (x, y) in song.analysis.iter().zip(expected_analysis) {
+        for (x, y) in song.analysis.to_vec().iter().zip(expected_analysis) {
             assert!(0.01 > (x - y).abs());
         }
     }
@@ -582,48 +697,28 @@ mod tests {
     #[test]
     fn test_analysis_distance() {
         let mut a = Song::default();
-        a.analysis = vec![
-            0.37860596,
-            -0.75483,
-            -0.85036564,
-            -0.6326486,
-            -0.77610075,
-            0.27126348,
-            -1.,
-            0.,
-            1.,
-        ];
+        a.analysis = Analysis::new([
+            0.16391512, 0.11326739, 0.96868552, 0.8353934, 0.49867523, 0.76532606, 0.63448005,
+            0.82506196, 0.71457147, 0.62395476, 0.69680329, 0.9855766, 0.41369333, 0.13900452,
+            0.68001012, 0.11029723, 0.97192943, 0.57727861, 0.07994821, 0.88993185,
+        ]);
 
         let mut b = Song::default();
-        b.analysis = vec![
-            0.31255,
-            0.15483,
-            -0.15036564,
-            -0.0326486,
-            -0.87610075,
-            -0.27126348,
-            1.,
-            0.,
-            1.,
-        ];
-        assert_eq!(a.distance(&b), 5.986180)
+        b.analysis = Analysis::new([
+            0.5075758, 0.36440256, 0.28888011, 0.43032829, 0.62387977, 0.61894916, 0.99676086,
+            0.11913155, 0.00640396, 0.15943407, 0.33829514, 0.34947174, 0.82927523, 0.18987604,
+            0.54437275, 0.22076826, 0.91232151, 0.29233168, 0.32846024, 0.04522147,
+        ]);
+        assert_eq!(a.distance(&b), 1.9469079)
     }
 
     #[test]
     fn test_analysis_distance_indiscernible() {
         let mut a = Song::default();
-        a.analysis = vec![
-            0.37860596,
-            -0.75483,
-            -0.85036564,
-            -0.6326486,
-            -0.77610075,
-            0.27126348,
-            -1.,
-            0.,
-            1.,
-        ];
-
+        a.analysis = Analysis::new([
+            1., 2., 3., 4., 5., 6., 7., 8., 9., 10., 11., 12., 13., 14., 15., 16., 17., 18., 19.,
+            20.,
+        ]);
         assert_eq!(a.distance(&a), 0.)
     }
 
@@ -640,6 +735,33 @@ mod tests {
             BlissError::DecodingError(String::from("No audio stream found.")),
         );
     }
+
+    #[test]
+    fn test_index_analysis() {
+        let song = Song::new("data/s16_mono_22_5kHz.flac").unwrap();
+        assert_eq!(song.analysis[AnalysisIndex::Tempo], 0.3846389);
+        assert_eq!(song.analysis[AnalysisIndex::Chroma10], -0.95968974);
+    }
+
+    #[test]
+    fn test_debug_analysis() {
+        let song = Song::new("data/s16_mono_22_5kHz.flac").unwrap();
+        assert_eq!(
+            "Analysis { Tempo: 0.3846389, Zcr: -0.849141, MeanSpectralCentroid: \
+            -0.75481045, StdDeviationSpectralCentroid: -0.8790748, MeanSpectralR\
+            olloff: -0.63258266, StdDeviationSpectralRolloff: -0.7258959, MeanSp\
+            ectralFlatness: -0.7757379, StdDeviationSpectralFlatness: -0.8146726\
+            , MeanLoudness: 0.2716726, StdDeviationLoudness: 0.25779057, Chroma1\
+            : -0.35661936, Chroma2: -0.63578653, Chroma3: -0.29593682, Chroma4: \
+            0.06421304, Chroma5: 0.21852458, Chroma6: -0.581239, Chroma7: -0.946\
+            6835, Chroma8: -0.9481153, Chroma9: -0.9820945, Chroma10: -0.95968974 } \
+            /* [0.3846389, -0.849141, -0.75481045, -0.8790748, -0.63258266, -0.\
+            7258959, -0.7757379, -0.8146726, 0.2716726, 0.25779057, -0.35661936, \
+            -0.63578653, -0.29593682, 0.06421304, 0.21852458, -0.581239, -0.946\
+            6835, -0.9481153, -0.9820945, -0.95968974] */",
+            format!("{:?}", song.analysis),
+        );
+    }
 }
 
 #[cfg(all(feature = "bench", test))]
